#ifndef GROUP_H_
#define GROUP_H_

#include "mesh.hpp"

#include <Eigen/Core>
#include <Eigen/Dense>
#include <vector>

// Compute an array of vertex indices whose distance
// to the source vertex is below a threshold.
// The input contains
// 	  mesh 		- mesh data
//    vid 		- Source vertex index.
//    dis_thres - Distance threshold.
// The output contains
//    vertex_indices - Neighboring vertices of vid.
void ExtractNeighbors(
	const Mesh& mesh,
	int vid,
	std::vector<int>& vertex_indices,
	double dis_thres);

// Compute an array of small regions each with an array
// of vertex indices. Additionally compute an array of
// pairs of indices for adjacent regions.
// The input contains
// 	  mesh 		- mesh data
//    dis_thres - Distance threshold for the region.
// The output contains
//    vertex_groups - An array of extracted regions.
//    group_pairs   - An array of adjacent region pairs.
void ExtractGroups(
	const Mesh& mesh,
	std::vector<std::vector<int> >& vertex_groups,
	std::vector<Eigen::Vector2i>& group_pairs,
	double dis_thres);

#endif