#include "io.hpp"

#include <fstream>
#include <strstream>
#include <vector>

using namespace Eigen;

void LoadOBJ(const char* filename, MatrixXd& V, MatrixXi& F)
{
	std::ifstream is(filename);
	char buffer[1024];
	std::vector<Vector3d> vertices;
	std::vector<Vector3i> faces;
	
	while (is.getline(buffer, 1024)) {
		std::strstream str;
		str << buffer;
		str >> buffer;
		if (strcmp(buffer, "v") == 0) {
			double x, y, z;
			str >> x >> y >> z;
			vertices.push_back(Vector3d(x, y, z));
		}
		else if (strcmp(buffer, "f") == 0) {
			Vector3i f;
			for (int i = 0; i < 3; ++i) {
				str >> buffer;
				int slash_num = 0;
				int l = strlen(buffer);
				for (int j = 0; j < l; ++j)
					slash_num += (buffer[j] == '/');
				if (slash_num == 0)
					sscanf(buffer, "%d", &f[i]);
				else
					sscanf(buffer, "%d/%s", &f[i], buffer);
				f[i] -= 1;
			}
			faces.push_back(f);
		}
	}

	V = Map<Matrix<double, Dynamic, Dynamic, RowMajor> >(
		(double*)vertices.data(),
		(int)vertices.size(),
		3);
	F = Map<Matrix<int, Dynamic, Dynamic, RowMajor> >(
		(int*)faces.data(),
		(int)faces.size(),
		3);
}

void SaveOBJ(const char* filename, const MatrixXd& V, const MatrixXi& F) {
	std::ofstream os(filename);
	for (int i = 0; i < V.rows(); ++i) {
		auto v = V.row(i);
		os << "v " << v[0] << " " << v[1] << " " << v[2] << "\n";
	}
	for (int i = 0; i < F.rows(); ++i) {
		auto f = F.row(i);
		os << "f " << f[0] + 1 << " " << f[1] + 1 << " " << f[2] + 1 << "\n";
	}
	os.close();
}

void SaveColoredOBJ(const char* filename,
	const MatrixXd& V,
	const MatrixXi& F,
	const MatrixXd& C) {
	std::ofstream os(filename);
	for (int i = 0; i < V.rows(); ++i) {
		Eigen::Vector3d v = V.row(i);
		Eigen::Vector3d c = C.row(i);
		os << "v " << v[0] << " " << v[1] << " " << v[2] << " "
		   << c[0] << " " << c[1] << " " << c[2] << "\n";
	}
	for (int i = 0; i < F.rows(); ++i) {
		auto f = F.row(i);
		os << "f " << f[0] + 1 << " " << f[1] + 1 << " " << f[2] + 1 << "\n";
	}
	os.close();
}
