#ifndef IO_H_
#define IO_H_

#include <Eigen/Core>

// Load the vertices and faces from the obj file.
// The input contains
//    filename - Input obj file path.
// The output contains
//    V - An array of vertices.
//    F - An array of faces.
void LoadOBJ(const char* filename, Eigen::MatrixXd& V, Eigen::MatrixXi& F);


// Store the vertices and faces as the obj file.
// The input contains
//    filename - Output obj file path.
//    V - An array of vertices.
//    F - An array of faces.
void SaveOBJ(
	const char* filename,
	const Eigen::MatrixXd& V,
	const Eigen::MatrixXi& F);


// Store the vertices, faces and colors as the obj file.
// The input contains
//    filename - Output obj file path.
//    V - An array of vertices.
//    F - An array of faces.
//    C - An array of vertex colors.
void SaveColoredOBJ(
	const char* filename,
	const Eigen::MatrixXd& V,
	const Eigen::MatrixXi& F,
	const Eigen::MatrixXd& C);

#endif