#include "group.hpp"
#include "io.hpp"
#include "mesh.hpp"
#include "primitive.hpp"
#include "region.hpp"
#include "subdivide.hpp"

int main(int argc, char** argv) {
	// Mesh data structure
	Mesh mesh;

	printf("********************************************\n");
	printf("Load from %s\n", argv[1]);
	LoadOBJ(argv[1], mesh.V, mesh.F);

	printf("Build half edges...\n");
	mesh.BuildQuadEdges();

	printf("Subdivide...\n");
	Subdivide(mesh, 1e-2);

	printf("Compute normals...\n");
	mesh.ComputeVertexNormals();

	// Region and relationships
	std::vector<std::vector<int> > vertex_groups;
	std::vector<Eigen::Vector2i> group_pairs;
	std::vector<Signature> signatures;

	printf("ExtractGroups...\n");
	ExtractGroups(mesh, vertex_groups, group_pairs, 3e-2);

	printf("Merge Regions...\n");
	MergeRegions(mesh, vertex_groups, group_pairs, signatures);
	RefineRegions(mesh, signatures, vertex_groups);

	// Export segmented model for visualization
	Eigen::MatrixXd C;
	mesh.ComputeVertexColors(vertex_groups, C);

	printf("Save to %s\n", argv[2]);
	SaveColoredOBJ(argv[2], mesh.V, mesh.F, C);	

	return 0;
}

