#ifndef REGION_H_
#define REGION_H_

#include "mesh.hpp"
#include "primitive.hpp"

#include <Eigen/Core>
#include <iostream>
#include <set>

// Merge an array of regions of the mesh.
// The input contains
//    mesh 			- Mesh data
//    vertex_groups - An array of extracted regions.
//    group_pairs   - An array of adjacent region pairs.
//    signatures    - An array of region signature
// The output contains
//    vertex_groups, group_pairs and the signatures for
//    the merged regions.
void MergeRegions(
	const Mesh& mesh,
	std::vector<std::vector<int> >& vertex_groups,
	std::vector<Eigen::Vector2i>& group_pairs,
	std::vector<Signature>& signatures);

// Refine the regions of the mesh.
// The input contains
//    mesh 			- Mesh data
//    signatures    - An array of region signature
//    vertex_groups - An array of extracted regions.
// The output contains
//    vertex_groups of the refined regions.
void RefineRegions(
	const Mesh& mesh,
	const std::vector<Signature>& signatures,
	std::vector<std::vector<int> >& vertex_groups);

#endif