# CS348A_Assignment
Computer Graphics: Geometric Modeling and Processing

## Dataset
Download the training and testing dataset as follow.
```
wget --no-check-certificate https://shapenet.cs.stanford.edu/media/minhyuk/spfn/data/spfn_traceparts.zip
unzip spfn_traceparts.zip
```

## Task 1: Implement a Data Loader
The first task is to implement a data loader.

In src/dataset.py, there are 5 tasks to complete the data loader.

1. Load the CSV file to get a list of filenames storing the data.
2. Read the file and parse the data.
3. Batch the data and store them into the memory cache.
4. Directly Load data from memory cache.
5. Shuffle the cached data at each new iteration.

In src/primitive, there is a function called extract_parameter_data_as_dict in cone.py, cylinder.py, plane.py and sphere.py.
It extracts the axes parameters for each instance as a dictionary, which will be used for axes supervision in the training process.
Each primitive type will allocate an array of Kx3.

1. For each primitive type, implement this function and fill the axis parameters information into the array at location i if i-th instance is with the same type.

### More instructions for implementation.
The most important task is the second one. We need to parse the file to get the information.
* P: input point positions (Nx3). It is noisy or ground truth depending on whether noisy is True
* normal_gt: ground truth point normals (Nx3).
* I_gt: ground truth per-point instance ID (Nx1).
* P_gt: ground truth point positions per instance (KxMx3), K is n_max_instances
* T_gt: ground truth per-instance primitive type (Kx1).
* instance: Not larger than K numbers of instance objects created from the meta data.

Each HDF5 file is a point cloud containing multiple primitive instances with different types. They are stored in a dictionary format:
```
{
    'gt_points'     : Point positions of N points (Nx3)
    'noisy_points'  : Point positions of N points with noises (Nx3)
    'gt_normals'    : Point normals of N points (Nx3)
    'gt_labels'     : Instance ID of N points (Nx1)
    '*_soup_[0-9]+' : {                                                 # Multiple keys, each for one primitive instance
        'gt_points'     : Point positions of M points for the instance
        .attrs['meta']  : Primitive type and parameters                 # use pickle.loads() to decode it to dictionary
    }
}
```
For the meta attributes, it contains a 'type' that specifies the primitive type, additionally with several keys about the corresponding parameters of the primitive. Simply call primitives.create_primitive_from_dict to create the instance.

The instance ID can be retrieved using primitives.primitive_name_to_id(instance.get_primitive_name()), which needs to be stored in T_gt.

After finishing the task, run
```
python test.py
```
to check whether the test succeeds. Comment test_compute_parameters since it is not implemented yet.

## Task 2: Implement the Network Architecture
The entry point for training the network is in train.py. It creates the dataset and call the train function in network.py. First compile the pointnet++ library by calling compiling script in src/pointnet_plusplus/utils/tf_ops/[operation]/...compile.sh.

### Create Input placeholder.
The main network graph is built in network constructor. During training, the input to network is the point cloud P and the ground truth data for supervision. The placeholders are specified as follow:
```
P : ? (batch) x ? (N) x 3                                     # Input Point cloud
gt_dict: {
    'instance_per_point' : ? (batch) x ? (N)                  # Ground truth instance ID
    'normal_per_point'   : ? (batch) x ? (N) x 3              # Ground truth point normals
    'type_per_instance'  : ? (batch) x K                      # Ground truth primitive types for each instance
    'points_per_instance': ? (batch) x K x ? (M) x 3          # Ground truth points for each instance
    'parameters' : {
        'plane_n'       : ? (batch) x K x 3                   # Ground truth plane normals
        'cone_axis'     : ? (batch) x K x 3                   # Ground truth cone axes
        'cylinder_axis' : ? (batch) x K x 3                   # Ground truth cylinder axes
    }
}
```
The parameters can be automatically created for different types of primitives by implementing the fill_gt_placeholders function in cone.py, cylinder.py, plane.py and sphere.py.

### Feed the data
During training, we feed the batched input data to the placeholders. Implement it in function "create_feed_dict" in src/network.py. For parameters, implement them in fill_gt_placeholders function in different primitive classes (cone.py, cylinder.py, plane.py and sphere.py).

### Prediction
Predict the point instance IDs, normals, primitive types, and primitive parameters. In architecture.py, implement the network architecture. Point features can be extracted using function "build_pointnet2_seg" from pointnet++ in src/pointnet_plusplus/architectures.py.

The features are passed to three separate fully connected layers to predict the membership matrix, point normals, and primitive types. You can use the fully connected layer called "fully_connected" from pointnet++ in src/pointnet_plusplus/utils/tf_utils.py.

Primitive parameters are estimated by calling each instance's compute_parameters function in cone.py, plane.py, cylinder.py. Implement these functions in different primitives, and output the primitive parameters as
* For cone, add 'cone_apex', 'cone_axis', and 'cone_half_angle' into the parameters dictionary.
* For cylinder, add 'cylinder_axis', 'cylinder_center', and 'cylinder_radius_squared' into the parameters dictionary
* For plane, add 'plane_n' and 'plane_c' (distance to origin) to the parameters dictionary.
* For sphere, add 'sphere_center' and 'sphere_radius_squared' to the parameters dictionary.

### Implement the losses
In src/evaluation.py, implement the losses starting from line 83. For primitive fitting and axis angle loss, implement compute_residue_loss and compute_parameter_loss for different primitives in cone.py, cylinder.py, plane.py and sphere.py.

### Unit test
Run src/test_parameters.py to verify that compute_parameters and compute_residue_loss are implemented correctly.
Run src/test_dataloader.py to verify that the data loader is implemented correctly.
Run src/test_loss.py to verify that the losses are implemented correctly.
Note that the test is just for some basic verification. It may not cover all the cases.
