from primitive import primitives
import numpy as np
import random
import os
import h5py
import pickle
import pandas
import re

def create_unit_data_from_hdf5(f, n_max_instances, noisy, fixed_order=False, check_only=False, shuffle=True):
    ############################
    # TODO Task 1.2: Load the following information from the HDF5
    # P: input point positions (Nx3). It is noisy or ground truth depending on whether noisy is True
    # normal_gt: ground truth point normals (Nx3).
    # I_gt: ground truth per-point instance ID (Nx1).
    # P_gt: ground truth point positions per instance (KxMx3), K is n_max_instances
    # T_gt: ground truth per-instance primitive type (Kx1).
    P = None
    P_gt = None
    normal_gt = None
    I_gt = None
    T_gt = None
    instances = []

    if shuffle:
        # random shuffle per-point information

    ############################

    result = {
        'P': P,
        'normal_gt': normal_gt,
        'P_gt': P_gt,
        'I_gt': I_gt,
        'T_gt': T_gt,
    }

    # Finally put in primitive parameters
    for primitive in primitives.all_primitive_classes:
        result.update(primitive.extract_parameter_data_as_dict(instances, n_max_instances))

    return result

class Dataset:
    def __init__(self, batch_size, n_max_instances, csv_path, noisy, fixed_order=False):
        self.batch_size = batch_size
        self.n_max_instances = n_max_instances
        self.fixed_order = fixed_order
        self.noisy = noisy

        ############################
        # TODO Task 1.1: Load the hdf5 file list and shuffle it if fixed_order = False
        self.hdf5_file_list = []
        ############################

        self.basename_list = [os.path.splitext(os.path.basename(p))[0] for p in self.hdf5_file_list]
        self.n_data = len(self.hdf5_file_list)
        self.first_iteration_finished = False
        self.data_matrix = {}

    def fetch_data_at_index(self, i):
        assert not self.first_iteration_finished
        path = self.hdf5_file_list[i]
        name = os.path.splitext(os.path.basename(path))[0]
        with h5py.File(path, 'r') as handle:
            data = create_unit_data_from_hdf5(handle, self.n_max_instances, noisy=self.noisy, fixed_order=self.fixed_order, shuffle=not self.fixed_order)
            assert data is not None # assume data are all clean

        return data

    def __iter__(self):
        self.current = 0
        ############################
        # TODO Task 1.5: At the beginning of each iteration, shuffle the data in self.data_matrix
        if not self.fixed_order and self.first_iteration_finished:
            pass
        ############################

        return self

    def __next__(self):
        if self.current >= self.n_data:
            if not self.first_iteration_finished:
                self.first_iteration_finished = True
            raise StopIteration()

        step = min(self.n_data - self.current, self.batch_size)
        assert step > 0
        self.last_step_size = step
        batched_data = {}

        if self.first_iteration_finished:
            ############################
            # TODO Task 1.4: Load the cached data from self.data_matrix to batched_data after the first iteration
            batched_data = None
            ############################
        else:
            data = []
            for i in range(step):
                data.append(self.fetch_data_at_index(self.current + i))

            ############################
            # TODO Task 1.3: Batch the loaded data for each key and cache into the data_matrix
            batched_data = None
            self.data_matrix = None
            ############################

        self.current += step
        return batched_data

    def get_last_batch_range(self):
        # return: [l, r)
        return (self.current - self.last_step_size, self.current)

    def get_last_batch_basename_list(self):
        assert self.fixed_order
        l, r = self.get_last_batch_range()
        return self.basename_list[l:r]

    def create_iterator(self):
        return self

def test():
    train_data = Dataset(
        batch_size=2, 
        n_max_instances=24, 
        csv_path='../traceparts_data/train_models.csv', 
        noisy=True, 
        fixed_order=True, 
    )

    step = 0
    for batch in train_data.create_iterator():
        fp = open('../test/test_data.pkl','rb')
        batch_gt = pickle.load(fp)
        for k in batch_gt.keys():
            if not k in batch:
                print('Key error: %s not exists'%(k))
                return False
            if batch_gt[k].shape != batch[k].shape:
                print('Data error: shape is inconsistent at %s'%(k))
                return False
            if np.max(np.abs(batch_gt[k] - batch[k])) > 1e-5:
                print(np.max(np.abs(batch_gt[k] - batch[k])))
                print('Data error: data is different from ground truth at %s'%(k))
                return False
        break
    print('Test success!')
    return True
if __name__ == "__main__":
    test()