from .util import *
import tensorflow as tf
class Plane:
    @staticmethod
    def get_primitive_name():
        return 'plane'

    @staticmethod
    def create_primitive_from_dict(d):
        assert d['type'] == 'plane'
        location = np.array([d['location_x'], d['location_y'], d['location_z']], dtype=float)
        axis = np.array([d['axis_x'], d['axis_y'], d['axis_z']], dtype=float)
        return Plane(n=axis, c=np.dot(location, axis))

    @staticmethod
    def extract_parameter_data_as_dict(primitives, n_max_instances):
        return {
            'plane_n_gt': np.zeros(dtype=float, shape=[n_max_instances, 3])
        }

    @staticmethod
    def insert_gt_placeholders(parameters_gt, n_max_instances):
        pass

    @staticmethod
    def fill_gt_placeholders(feed_dict, parameters_gt, batch):
        pass

    @staticmethod
    def compute_parameters(feed_dict, parameters):
        pass

    @staticmethod
    def compute_residue_loss(parameters, P_gt, matching_indices):
        pass

    @staticmethod
    def compute_parameter_loss(parameters_pred, parameters_gt, matching_indices, angle_diff):
        pass

    def __init__(self, n, c, center=None, x_axis=None, y_axis=None, x_range=[-1, 1],  y_range=[-1, 1]):
        if type(n) is not np.ndarray:
            print('Normal {} needs to be a numpy array!'.format(n))
            raise
        # Plane is defined by {p: n^T p = c}, where the bound is determined by xy_range w.r.t. center
        if center is None:
            center = n * c
        self.n = n / np.linalg.norm(n)
        self.c = c
        self.center = center
        self.x_range = x_range
        self.y_range = y_range

        # parameterize the plane by picking axes
        if x_axis is None or y_axis is None:
            ax_tmp = make_rand_unit_vector()
            self.x_axis = normalized(np.cross(ax_tmp, self.n))
            self.y_axis = normalized(np.cross(self.n, self.x_axis))
        else:
            self.x_axis = x_axis
            self.y_axis = y_axis


