from .cone import Cone
from .cylinder import Cylinder
from .plane import Plane
from .sphere import Sphere

NAME_TO_PRIMITIVE_DICT = {
    'plane': Plane,
    'sphere': Sphere,
    'cylinder': Cylinder,
    'cone': Cone,
}

primitive_name_to_id_dict = {}
all_primitive_classes = []

def create_primitive_from_dict(d):
    if not d['type'] in NAME_TO_PRIMITIVE_DICT:
        return None
    else:
        return NAME_TO_PRIMITIVE_DICT[d['type']].create_primitive_from_dict(d)

def get_all_primitives_classes():
    return all_primitive_classes

def primitive_name_to_id(name):
    return primitive_name_to_id_dict[name]

def get_n_registered_primitives():
    return len(all_primitive_classes)
    
def register_primitives(primitive_name_list):
    # Must be called once before everything
    global all_primitive_classes, primitive_name_to_id_dict
    all_primitive_classes = []
    primitive_name_to_id_dict = {}

    for idx, name in enumerate(primitive_name_list):
        primitive = NAME_TO_PRIMITIVE_DICT[name]
        all_primitive_classes.append(primitive)
        primitive_name_to_id_dict[name] = idx

    print('Registered ' + ','.join(primitive_name_list))

def get_list_of_primitives():
    return ['plane','sphere','cylinder','cone']

if get_n_registered_primitives() == 0:
    register_primitives(get_list_of_primitives())