import os, sys

from network import Network
from dataset import Dataset
from primitive.util import *
from primitive import primitives

import random
import tensorflow as tf
import numpy as np
import os
import json
import argparse

if __name__ == '__main__':
    tf.set_random_seed(1234)
    np.random.seed(1234)
    random.seed(1234)
    register_custom_svd_gradient()

    parser = argparse.ArgumentParser()
    parser.add_argument('--test', action='store_true', help='Run network in test time')
    parser.add_argument('--model_dir', type=str, default='results/model')
    parser.add_argument('--pred_dir', type=str, default='results/test_pred')
    parser.add_argument('--val_pred_dir', type=str, default='results/val_pred')
    parser.add_argument('--log_dir', type=str, default='results/log')
    parser.add_argument('--train_data', type=str, default='../../traceparts_data/train_models.csv')
    parser.add_argument('--val_data', type=str, default='../../traceparts_data/test_models.csv')
    parser.add_argument('--test_data', type=str, default='../../traceparts_data/test_models.csv')
    args = parser.parse_args()

    is_testing = True if args.test else False
    batch_size = 16
    n_max_instances = 24

    tf_conf = tf.ConfigProto()
    tf_conf.allow_soft_placement = True
    tf_conf.gpu_options.allow_growth = True

    in_model_dir = args.model_dir
    ckpt = tf.train.get_checkpoint_state(in_model_dir)
    should_restore = (ckpt is not None) and (ckpt.model_checkpoint_path is not None)

    print('Building network...')
    net = Network(n_max_instances=n_max_instances)
    with tf.Session(config=tf_conf, graph=net.graph) as sess:
        if should_restore:
            print('Restoring ' + ckpt.model_checkpoint_path + ' ...')
            tf.train.Saver().restore(sess, ckpt.model_checkpoint_path)
        else:
            assert not is_testing
            print('Starting a new training...')
            sess.run(tf.global_variables_initializer())

        print('Loading data...')
        if is_testing:
            # batch testing
            test_data = Dataset(
                batch_size=batch_size, 
                n_max_instances=n_max_instances, 
                csv_path=args.test_data, 
                noisy=True, 
                fixed_order=True, 
            )
            net.predict_and_save(
                sess,
                dset=test_data,
                save_dir=args.pred_dir,
            )
        else:
            train_data = Dataset(
                batch_size=batch_size, 
                n_max_instances=n_max_instances, 
                csv_path=args.train_data, 
                noisy=True, 
                fixed_order=False, 
            )
            val_data = Dataset(
                batch_size=batch_size, 
                n_max_instances=n_max_instances, 
                csv_path=args.val_data, 
                noisy=True, 
                fixed_order=True, 
            )
            net.train(
                sess, 
                train_data=train_data, 
                val_data=val_data, 
                n_epochs=25, 
                val_interval=2000,
                snapshot_interval=500,
                model_dir=args.model_dir,
                log_dir=args.log_dir,
                val_pred_dir=args.val_pred_dir,
            )

